﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using Microsoft.Glee.Drawing;
using Microsoft.Glee.GraphViewerGdi;

namespace HeuristicLab.EvolutionaryTracking {
  [View("GenealogyGraphView")]
  [Content(typeof(GenealogyGraph), IsDefaultView = true)]
  public sealed partial class GenealogyGraphView : ItemView {
    private readonly GViewer _gViewer;

    public new GenealogyGraph Content {
      get { return (GenealogyGraph)base.Content; }
      set { base.Content = value; }
    }

    public GenealogyGraphView()
      : base() {
      InitializeComponent();
      _gViewer = new GViewer() { Dock = DockStyle.Fill };
      this.Controls.Add(_gViewer);
    }

    protected override void DeregisterContentEvents() {
      // TODO: Deregister your event handlers here
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // TODO: Register your event handlers here
    }

    #region Event Handlers (Content)
    // TODO: Put event handlers of the content here
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        // TODO: Add code when content has been changed and is null
      } else {
        // TODO: Add code when content has been changed and is not null
        if (InvokeRequired) {
          Invoke(new EventHandler(Content_GraphChanged), this, null);
        } else {
          SetEnabledStateOfControls();
          Content_GraphChanged(this, null);
        }
      }
    }

    private void Content_GraphChanged(object sender, EventArgs e) {
      var graph = Content;
      var nodes = graph.Values;
      var gleeGraph = new Microsoft.Glee.Drawing.Graph("PopulationGraph") {
        GraphAttr = {
          Backgroundcolor = Microsoft.Glee.Drawing.Color.White,
          NodeAttr = { Shape = Shape.Circle, Label = String.Empty, Padding = 40 },
          AspectRatio = 2
        }
      };

      foreach (var node in nodes) {
        var n = gleeGraph.AddNode(node.Id); // Glee does not actually add a new node if the node already exists in the graph
        n.Attr.Fillcolor = new Microsoft.Glee.Drawing.Color((byte)((1 - node.Quality) * 255), (byte)(255 * node.Quality), 0);
        n.Attr.Color = new Microsoft.Glee.Drawing.Color((byte)((1 - node.Quality) * 255), (byte)(255 * node.Quality), 0);
        n.Attr.Label = node.Label;
        n.Attr.Fontsize = 8;
        n.Attr.FontName = "Consolas";
        if (node.IsElite) {
          n.Attr.Color = Microsoft.Glee.Drawing.Color.DodgerBlue;
          n.Attr.Shape = Shape.Box;
        }

        if (node.OutEdges == null) continue;
        foreach (var arc in node.OutEdges) {
          var edge = gleeGraph.AddEdge(node.Id, arc.Target.Id);
          edge.Attr.LineWidth = 1;
          byte r = (byte)((edge.SourceNode.Attr.Color.R + edge.TargetNode.Attr.Color.R) / 2);
          byte g = (byte)((edge.SourceNode.Attr.Color.G + edge.TargetNode.Attr.Color.G) / 2);
          byte b = (byte)((edge.SourceNode.Attr.Color.B + edge.TargetNode.Attr.Color.B) / 2);
          edge.Attr.Color = new Microsoft.Glee.Drawing.Color(r, g, b);
        }
      }
      _gViewer.Graph = gleeGraph;
      _gViewer.Refresh();
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      // TODO: Enable or disable controls based on whether the content is null or the view is set readonly
    }

    #region Event Handlers (child controls)
    // TODO: Put event handlers of child controls here.
    #endregion
  }

}